@@ -45,12 +45,14 @@
 #define OSSO_GDL_OBJECT  "/org/maemo/"OSSO_GDL_NAME
 #define OSSO_GDL_IFACE   "org.maemo."OSSO_GDL_NAME
 
+#define AUTO_PAUSE_TIME_MULTIPLIER 5
 
 HD_DEFINE_PLUGIN_MODULE (GpsDataLoggerWidget, gps_data_logger_widget, HD_TYPE_HOME_PLUGIN_ITEM)
 
-static void
-update_status_and_buttons (GpsDataLoggerWidget * self);
-
+static void update_status_and_buttons (GpsDataLoggerWidget * self);
+static gboolean manual_on_stop (gpointer data);
+static void on_stop(LocationGPSDControl *control, gpointer data);
+	
 /* -------------------------------------------------------------------- */
 /* Settings
  */
@@ -157,6 +159,25 @@ on_settings_dialog_response (GtkDialog *dialog,
             save_needed = TRUE;
 	    self->settings->interval = new_int;
 	}
+	new_value = hildon_check_button_get_active(
+		    HILDON_CHECK_BUTTON(self->chb_auto_pause));
+        if (self->settings->auto_pause != new_value) {
+	    save_needed = TRUE;
+	    self->settings->auto_pause = new_value;
+	}
+        new_int = hildon_picker_button_get_active(
+		  HILDON_PICKER_BUTTON(self->pkr_pause_speed_threshold));
+	if (self->settings->pause_speed_threshold != new_int) {
+            save_needed = TRUE;
+	    self->settings->pause_speed_threshold = new_int;
+	}
+        new_int = hildon_picker_button_get_active(
+		  HILDON_PICKER_BUTTON(self->pkr_pause_time_threshold));
+	if (self->settings->pause_time_threshold != new_int) {
+            save_needed = TRUE;
+	    self->settings->pause_time_threshold = new_int;
+	}
+	
 	if (g_ascii_strcasecmp(self->settings->log_folder,
                                hildon_button_get_value(
 			       HILDON_BUTTON(self->btn_path))) != 0) {
@@ -181,8 +202,10 @@ settings_dialog_new (GpsDataLoggerWidget * self)
 {
     GtkWidget *dialog;
     GtkWidget *vbox;
-    GtkWidget *selector;
+    GtkWidget *selector, *speed_selector, *time_selector;
     GtkWidget *panarea;
+    gchar *text;
+    int i;
 
     dialog = gtk_dialog_new_with_buttons (_("GPS Data Logger Widget Settings"),
 					  NULL, /* parent */
@@ -270,6 +293,62 @@ settings_dialog_new (GpsDataLoggerWidget * self)
                         self->pkr_interval, TRUE, TRUE, 5);
     hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector),
                                       0, self->settings->interval);
+
+    /* Auto pause */
+    self->chb_auto_pause = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT);
+    gtk_button_set_label (GTK_BUTTON (self->chb_auto_pause), _("Auto pause enabled"));
+    hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->chb_auto_pause),
+		    		self->settings->auto_pause);
+    gtk_box_pack_start (GTK_BOX (vbox),
+		        self->chb_auto_pause, TRUE, TRUE, 5);
+    /* speed threshold */
+    self->pkr_pause_speed_threshold = hildon_picker_button_new
+	               (HILDON_SIZE_FINGER_HEIGHT,
+		        HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+    gtk_button_set_alignment (GTK_BUTTON (self->pkr_pause_speed_threshold), 
+		    		0.0, 0.5);
+    hildon_button_set_title (HILDON_BUTTON (self->pkr_pause_speed_threshold),
+		               _("Pause if speed is less than"));
+    speed_selector = hildon_touch_selector_new_text ();
+    text = g_malloc0(10);
+    for (i = 1; i <= 30; i++ ) {
+        g_sprintf(text, "%d km/h", i);
+	hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (speed_selector),
+			text);
+    }
+    g_free(text);
+    hildon_picker_button_set_selector(HILDON_PICKER_BUTTON
+             (self->pkr_pause_speed_threshold), 
+	     HILDON_TOUCH_SELECTOR (speed_selector));
+    gtk_box_pack_start (GTK_BOX (vbox),
+                        self->pkr_pause_speed_threshold, TRUE, TRUE, 5);
+    hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (speed_selector),
+                                      0, self->settings->pause_speed_threshold);
+    /* interval threshold */
+    self->pkr_pause_time_threshold = hildon_picker_button_new
+	               (HILDON_SIZE_FINGER_HEIGHT,
+		        HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+    gtk_button_set_alignment (GTK_BUTTON (self->pkr_pause_time_threshold), 
+		    		0.0, 0.5);
+    hildon_button_set_title (HILDON_BUTTON (self->pkr_pause_time_threshold),
+		               _("For at least"));
+    time_selector = hildon_touch_selector_new_text ();
+    text = g_malloc0(25);
+    for (i = 1; i <= 25; i++ ) {
+        g_sprintf(text, "%d seconds", i * AUTO_PAUSE_TIME_MULTIPLIER);
+	hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (time_selector),
+			text);
+    }
+    g_free(text);
+    hildon_picker_button_set_selector(HILDON_PICKER_BUTTON
+             (self->pkr_pause_time_threshold), 
+	     HILDON_TOUCH_SELECTOR (time_selector));
+    gtk_box_pack_start (GTK_BOX (vbox),
+                        self->pkr_pause_time_threshold, TRUE, TRUE, 5);
+    hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (time_selector),
+                                      0, self->settings->pause_time_threshold);
+    
+    
     /* Log */
     gtk_box_pack_start (GTK_BOX (vbox),
                         gtk_label_new (_("Log")),
@@ -431,7 +510,7 @@ debuglog_close(GpsDataLoggerWidget * self)
 static void
 debuglog_start(GpsDataLoggerWidget * self)
 {
-    gchar *path;
+    /* gchar *path; */
     time_t current;
     struct tm *tp;
 
@@ -442,7 +521,9 @@ debuglog_start(GpsDataLoggerWidget * self)
     /* Name the log file using the current time */
     current = time(NULL);
     tp = localtime(&current);
-    path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d.log", 
+    if (self->path != NULL)
+	g_free(self->path);
+    self->path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d.log", 
 		           self->settings->log_folder,
 		    	   tp->tm_year + 1900,
 		   	   tp->tm_mon + 1,
@@ -450,8 +531,8 @@ debuglog_start(GpsDataLoggerWidget * self)
 		   	   tp->tm_hour,
 		   	   tp->tm_min,
 		   	   tp->tm_sec );
-    self->logfile = fopen(path, "w");
-    g_free(path);
+    self->logfile = fopen(self->path, "w");
+    /* g_free(path); */
     
 }
 #endif
@@ -464,6 +545,10 @@ logfile_close(GpsDataLoggerWidget * self)
         fclose(self->file);
         self->file = NULL;
     }
+    if (self->path) {
+        g_free(self->path);
+	self->path = NULL;
+    }
 }
 
 static void
@@ -537,6 +622,7 @@ update_status_and_buttons (GpsDataLoggerWidget * self)
 	case APP_STATE_STARTING:
 		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
 				           self->gps_refresh_icon);
+		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
 		self->rec_button_enabled = FALSE;
 		self->stop_button_enabled = TRUE;
 #ifndef __arm__
@@ -546,14 +632,21 @@ update_status_and_buttons (GpsDataLoggerWidget * self)
 	case APP_STATE_RECORDING:
 		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
 				           self->gps_on_icon);
+		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
 		self->pause_button_visible = TRUE;
 		self->rec_button_enabled = TRUE;
 		self->stop_button_enabled = TRUE;
 		break;
 	case APP_STATE_PAUSED:
+		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
 		self->pause_button_visible = FALSE;
 		break;
+	case APP_STATE_AUTO_PAUSED:
+		gtk_image_set_from_pixbuf (GTK_IMAGE(self->auto_pause_status),
+				           self->pause_icon);
+		break;
 	case APP_STATE_STOPPING:
+		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
 		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
 				           self->gps_off_icon);
 		self->pause_button_visible = FALSE;
@@ -564,6 +657,7 @@ update_status_and_buttons (GpsDataLoggerWidget * self)
 	default:
 		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
 				           self->gps_off_icon);
+		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
 		self->pause_button_visible = FALSE;
 		self->rec_button_enabled = TRUE;
 		self->stop_button_enabled = FALSE;
@@ -638,6 +732,15 @@ handle_button_press (GtkWidget *button,
             location_gpsd_control_start(self->control);
 	}
     } else if (button == self->button_stop) {
+	if (self->app_state == APP_STATE_PAUSED) {
+	    /* If the state is changing from paused->stopped
+	     * then the gps is already stopped, so no on_stop
+	     * signal will come. Set a timer and call it. */
+	    g_timeout_add_seconds(2,
+			          manual_on_stop,
+				  data);
+
+	}
 	self->app_state = APP_STATE_STOPPING;
 #ifndef __arm__
 	g_print(_(" new state: stoping\n"));
@@ -841,6 +944,14 @@ on_start(LocationGPSDControl *control, gpointer data)
 #endif
 }
 
+static gboolean
+manual_on_stop(gpointer data)
+{
+    /* call on_stop manually */
+    on_stop(NULL, data);
+    return FALSE;
+}
+
 static void
 on_stop(LocationGPSDControl *control, gpointer data)
 {
@@ -855,12 +966,19 @@ on_stop(LocationGPSDControl *control, gpointer data)
     }
 #endif
 	if (self->app_state == APP_STATE_STOPPING) {
+	    if (self->path) {
+        	hildon_banner_show_informationf(NULL,
+			                       NULL,
+		        	               _("on_stop, going to IDLE state\nLog saved to \"%s\""),
+					       self->path);
+	    } else {
         	hildon_banner_show_information(NULL,
 			                       NULL,
 		        	               _("on_stop, going to IDLE state"));
-		self->app_state = APP_STATE_IDLE;
+	    }
+	    self->app_state = APP_STATE_IDLE;
 #ifndef __arm__
-                g_print(_("on_stop : calling update_status_and_buttons\n"));
+            g_print(_("on_stop : calling update_status_and_buttons\n"));
 #endif
 #ifdef DEBUG
             if (self->logfile) {
@@ -886,10 +1004,16 @@ on_changed(LocationGPSDevice *device, gpointer data)
 
         if (device->fix) {
 		if (((self->app_state == APP_STATE_STARTING) ||
-		     (self->app_state == APP_STATE_RECORDING)) &&
+		     (self->app_state == APP_STATE_RECORDING) ||
+		     (self->app_state == APP_STATE_AUTO_PAUSED)) &&
 		    (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET) &&
 		    (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET)) {
-			if (self->app_state == APP_STATE_STARTING) {
+			    /* Shall we stay in auto_paused state ? */
+			if (((self->app_state == APP_STATE_AUTO_PAUSED) &&
+			     (device->fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) &&
+			     (device->fix->speed >= self->settings->pause_speed_threshold) ) ||
+			   /* or just started */
+			   (self->app_state == APP_STATE_STARTING)) {
 			    hildon_banner_show_information(NULL,
                                 NULL,
                                 _("on_changed, going to RECORDING state"));
@@ -900,7 +1024,28 @@ on_changed(LocationGPSDevice *device, gpointer data)
 #endif
                             update_status_and_buttons(self);
 	                    self->points_recorded_in_current_segment = 0;
+			    self->auto_pause_time_mark = 0;
+			} else if ((self->app_state == APP_STATE_RECORDING) &&  /* self->app_state == APP_STATE_STARTING  or keep auto_pause? */
+				   (self->settings->auto_pause) &&
+				   (device->fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) &&
+				   (device->fix->speed < self->settings->pause_speed_threshold)) {
+			    /* RECORDING, shall we auto_pause? */
+				if ((self->auto_pause_time_mark == 0) &&
+				    (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET)) {
+					self->auto_pause_time_mark = device->fix->time;
+				} else if (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET) {
+					if ((device->fix->time - self->auto_pause_time_mark) >=
+					    (self->settings->pause_time_threshold * AUTO_PAUSE_TIME_MULTIPLIER)) {
+					    /* auto_pause speed & time thresholds reached, do auto_pause */
+					    self->app_state = APP_STATE_AUTO_PAUSED;
+					    update_status_and_buttons(self);
+					    return;
+					}
+				}
+					
+			    
 			}
+			
 			tme = device->fix->time;
 			tp = gmtime(&tme);
 			g_snprintf(string_buf, 256, "<trkpt lat=\"%f\" lon=\"%f\">\n",
@@ -935,7 +1080,7 @@ on_changed(LocationGPSDevice *device, gpointer data)
 					device->satellites_in_use);
 			g_fprintf(self->file, "</trkpt>\n");
 			self->points_recorded_in_current_segment += 1;
-		}
+		} /* STARING & RECORDING & has coordinates */
 #ifndef __arm__
                 if (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET)
                         g_printf("%s:%s: time = %f\n",
@@ -977,8 +1122,10 @@ on_changed(LocationGPSDevice *device, gpointer data)
                                 device->fix->epd,
                                 device->fix->eps,
                                 device->fix->epc);
-        }
+#endif
+        } /* device->fix */
 
+#ifndef __arm__
         g_printf("%s:%s: Satellites in view: %d\n",
                         __FILE__, __func__,
                         device->satellites_in_view);
@@ -994,8 +1141,8 @@ on_changed(LocationGPSDevice *device, gpointer data)
                 if (device->cell_info->flags & LOCATION_CELL_INFO_WCDMA_CELL_INFO_SET)
                         g_printf( "Mobile Coutry Code WCDMA: %d\n",
                                         device->cell_info->wcdma_cell_info.mcc);
+        } 
 #endif
-        }
 }
 
 static void
@@ -1239,6 +1386,7 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 			_("osso_initialize failed"));
     }
     self->file = NULL;
+    self->path = NULL;
 #ifdef DEBUG
     self->logfile = NULL;
 #endif
@@ -1284,13 +1432,15 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
     gtk_widget_set_app_paintable(self->button_stop, TRUE);
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(self->button_stop), FALSE);
 
-    self->gps_status = gtk_image_new();    
+    self->gps_status = gtk_image_new();
+    self->auto_pause_status = gtk_image_new();
     status_text = gtk_label_new(_("Status: "));
 
-    hbox_status = gtk_hbox_new (FALSE, 2);
+    hbox_status = gtk_hbox_new (FALSE, 3);
     hbox_buttons = gtk_hbox_new (FALSE, 2);
     gtk_box_pack_start (GTK_BOX (hbox_status), status_text, FALSE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_status), self->gps_status, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox_status), self->auto_pause_status, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_rec, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_stop, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), hbox_status, FALSE, FALSE, 0);
